function fit = MyFunctionsfunc(x, fun0,opt_f)
global opt_f % Just for compatibility with CEC 2005
% Benchmark functions 
% INPUTS:
% -  x is a D-element row vector representing the candidate solution we want
%	 to evaluate, where D is the problem dimension
% -  fun0 is the index of the function of interest
% -  ~ is for opt_f, the minimum function value to try to reach. Not used here
%				but kept for consistency with some other benchmarks

% OUTPUT:
% -  fit is the fitness of x (i.e. fit = f(x))
global a % Just for tests
global x0 middle triangle  structure smooth  % Specific to functions built for difficulty analysis
global sunny
global xSol
global  nbParabD localMinF sizeA  sameMinF indParab % For Multi-paraboloid
global Gr  % For graph coloring
	[~,D]=size(x);
	x=x';
  
    if fun0 >1000 && fun0 <2151 % Lennard-Jones
    	fun=5;
    else fun=fun0;
    end

switch fun
  case -6 % Biparaboloid

    %{
   persistent xSol1
   persistent xSol2
   persistent fmax
   persistent shift
    %}
   xSol1(1)=1/4;
   xSol2(1)=3/4;
   if D>1
   for d=2:D
     xSol1(d)=1/2;
     xSol2(d)=1/2;
   end
   end
   
   fmax=1/16+(D-1)/4;
   shift=0.002;
   
   xSol1=xSol1';
   xSol2=xSol2';
   dx1=x-xSol1;
   dx2=x-xSol2;

  dist1=norm(dx1);
	dist2=norm(dx2);
   
   if dist1<dist2
    f=dist1^2;
    fit=f/fmax;
   else
   f=dist2^2;
     f=f+shift;
     fit=shift+(1-shift)*f/fmax;
   end
 
  case -5 % Paraboloid [0,1]^D => [0,1], for calibration
    %						Solution on (0.5)*
		fit=0;
    mid=0.5;
		for d=1:D
			fit=fit+(x(d)-mid)^2;
		end
    fit=4*fit/D;
    
  case -51 % Shifted paraboloid (sphere)
    %fit=x(1)^2+x(2)^2;
fit=(x(1)-0.9)^2+(x(2)+0.9)^2;
%{
    fit=0;
    for d=1:D
    fit=fit+x(d)^2;
    end
    %}
    %{
    %sh1=-0.9; sh2=0.9;
    sh1=0; sh2=0;
    fit=0;
    D1=max(1,floor(D/2));
    D2=D-D1;
    
    for d=1:D1
      fit=fit+(x(d)-sh1)^2;
    end
    
    for d=D1+1:D2
      fit=fit+(x(d)-sh2)^2;
    end
    %}
		case -4 % Needle (Parabola with plateau around)
		posMin=0.3*ones(1,D);
		width=0.001;
		r= pdist([x';posMin],'euclidean');
		if r<=width
			fit=r^2;
		else
			fit=width^2;
		end
	
		case -3 % Picece-wise linear
		if ~exist('a','var') a=0.3; end
		if x<a
			fit=(a-x)/a;
    else
			fit=(x-a)/(1-a);
		end

		case -2 % Line
		fit=1-x;
		%fit=x;
		
		case -1 % Impossible Plateau
		fit=1;

		case 0 % Paraboloid on [0,2D]*
%						Solution on (1, 2, 3, ..., D)
		fit=0;
		for d=1:D
			fit=fit+(x(d)-d)^2;
     % fit=fit+x(d)^2;
		end

    case 1 % Alpine. Minimum (1,2,...,D)
    	for d=1:D 
    		xd(d)=x(d)-d; 
        %xd(d)=x(d)-xSol(d);
      end
 
        fit = sum(abs(xd.*sin(xd)) + 0.1*abs(xd));
  
    case 2 % Tripod. Minimum (0, -50)		     			
    x1 = x(1) ; 
    x2 = x(2);
      
if x2<0
      fit=abs(x1)+abs(x2+50);
else
	 	if x1 < 0 
 			fit=1 + abs(x1+50) + abs(x2- 50);
		else 
			fit=2 + abs(x1-50) + abs(x2-50);
		end
end
     	case 3 % Gear train
     	z=1.0/6.931;
     	fit=z-x(1)*x(2)/(x(3)*x(4));
  		fit=fit*fit;
  
  		case 4 % Pressure vessel
  		fit=0.6224*x(1)*x(3)*x(4) + 1.7781*x(2)*x(3)*x(3) ...
  				+ x(1)*x(1)*(3.1661*x(4) +19.84*x(3));
  				
  		% Takes constraints into account (penalty method)
  		g(1)=0.0193*x(3)-x(1);
  		g(2)=0.00954*x(3)-x(2);
  		g(3)=1296000-pi*x(3)*x(3)*(x(4)+4*x(3)/3);
  	
  		pw=10^10;
  		if g(1)>0 c= 1+pw*g(1);  fit=fit*c*c; end
			if g(2)>0 c=1+pw*g(2); fit=fit*c*c;  end
			%if g(2)>0 c=1+g(2); fit=fit*c*c;  end
			if g(3)>0 c=1+pw*g(3); fit=fit*c*c;  end
			%if g(3)>0 c=1+g(3); fit=fit*c*c;  end
			
	
			case 5 % Lennard-Jones
			fit=lennardJones(x);
			
			case 6 % Flash
			c1=0.1;  
    c2=0.5;
    u1=x(1);
    if u1<2*c1 | u1>5*c1  fit=c2;
    else
      u2=u1-2*c1;
      if u2<=c1 fit=c2*(1-u2/c1);
      else
        u3=u1-3*c1;
        if u3<c1  fit=u3/c1;
        else
          u4=u1-4*c1;
          fit=1-c2*u4/c1;
        end
      end
    end
			
			case 7 % Comb
			c1=10; 
    c2=1;
    u=x(1);
    fit=1+sin(c1*u) +u/c1;
    fit=min(fit,c2);
			
			case 8 % Brush
			c1=10; c2=1;
    fit=0;
    for d=1:D
      xs=x(d)-d;
      fit=fit+abs(xs*sin(c1*xs))+abs(xs)/c1 ;
    end
    fit=min(fit,c2);
    
    case 9 % TSP6
    fit=TSP6eval(x); 
    
    case 91 % TSP
    fit=TSPeval(x); 
    
    case 10 % Canyon  on [0,10]
    border=9.99;
    if x<border
    	fit=0.8+x*0.2/border;
    else
    	fit=(10-x)/(10-border);
    end
    
    case 11 % Six Hump Camel Back
       x1=x(1); x2=x(2); 
       fit=	(4-2.1*x1^2+x1^4/3)*x1^2+x1*x2+(-4+4*x2^2)*x2^2;
       
    case 12 % Hartman 3D
    fit=hart3(x);
    
    case 13 % Hartman 6D
    fit=hart6(x); 
    
    
   case 14 % Griewank 
   s=0; p=1;
   for d=1:D
   	s=s+x(d)^2;
   	p=p*cos(x(d)/sqrt(d));
   end
   fit=1+s/4000-p;

        
  case 15 % Sunnyspell, for D=2
    fit=sunnyspell(sunny,x);
    
  case 16 % Multi-paraboloid
    fit=multiparaboloid(x,nbParabD,localMinF, sizeA,  sameMinF, indParab);
    
  case 17 % Gas transmission compressor design problem
    x1=x(1); x2=x(2); x3=x(3);
    fit= (8.61*10^5) *sqrt(x1) *x2 *(x3^(-2/3)) *((x2^2-1)^(-1/2))...
      + (3.69*10^4) *x3...
      + (7.72*10^8) *(x2^0.219)/x1...
      -(765.43*10^6)/x1;
    %fit=abs(fit);
    
  case 18 % Three cubes
%{
 Looking for integers (a b c) so that Ntarget=a^3 + b^3 +c^3=(a,b,c)^3
Examples:
 216 = (3, 4, 5)^3 (and of course (6,-2, 2) because 216=6^3
 2001 = ( 10, 10, 1)^3
 2017 = (11, 7, 7)^3
 33 = (8 866 128 975 287 528, -8 778 405 442 862 239, -2 736 111 468 807 040)^3
%}

      Ntarget=33; %33; % Should be consistent with maxi in get_fun-info
      fit=x(1)^3 + x(2)^3 + x(3)^3;
      fit=abs(fit-Ntarget);
  
      case 10000 % Artificial functions (see Landscapes)
    fit=valuex(x',x0,middle, triangle,structure,smooth);

case 19 % For distance between hesitant Pythagorean fuzzy sets
    m1=x(1); m2=x(2);
 n1=x(3); n2=x(4);
 p1=x(5); p2=x(6);

    fit=abs(m1^2-n1^2) + abs(m2^2-n2^2);
    fit=fit+abs(n1^2-p1^2) + abs(n2^2-p2^2);
    fit=fit-abs(m1^2-p1^2) - abs(m2^2-p2^2);

    penalty=0;
    if m1^2+m2^2>1  penalty=10+penalty; end
    if n1^2+n2^2>1  penalty=10+penalty; end
    if p1^2+p2^2>1  penalty=10+penalty; end
      
    fit=fit+penalty;
    
  case 20 % Test x*sin*sqrt 
    fit=0;
    for d=1:D
      xd=x(d);
      fit=fit-xd^2*sin(sqrt(abs(xd)));
    end
    
    case 27 % Rastrigin
        fit = 10*D + sum(x.^2 - 10*cos(2.*pi.*x));
    
     case 300 % Shifted Rosenbrock
      shift = [ 81.023200 -48.395000 19.231600 ...
      -2.5231000 70.433800 47.177400 -7.8358000 ...
      -86.669300 57.853200 -9.9533000 20.777800 ...
      52.548600 75.92630  42.877300 -58.272000 ...
      -16.972800 78.384500 75.042700 -16.151300 ...
      70.856900 -79.579500 -26.483700 56.369900 ...
      -88.224900 -64.999600 -53.502200 -54.230000 ...
      18.682600 -41.006100 -54.213400 ...
      ];
  
      f = 0;
        for i = 1:D-1
          zi=x(i)-shift(i);
          zi1=x(i+1)-shift(i+1);
            f = f + 100*((zi1 - zi^2)^2) + (1 - zi)^2;
        end
        fit = f;
        
         case 301 %Rosenbrock
      f = 0;
        for i = 1:D-1
          zi=x(i);
          zi1=x(i+1);
            f = f + 100*((zi1 - zi^2)^2) + (1 - zi)^2;
        end
        fit = f;
      
  case 31 % Ackley
           fit = -20*exp(-0.2*sqrt(sum(x.^2)/D)) - exp(sum(cos(2*pi .*x))/D) + 20 + exp(1);

        
        case 52 % Shifted Schwefel 1.2 
      offset= [ 3.5626700e+001, -8.2912300e+001, -1.0642300e+001, ...
      -8.3581500e+001, 8.3155200e+001, 4.7048000e+001, -8.9435900e+001,...
      -2.7421900e+001, 7.6144800e+001, -3.9059500e+001, 4.8885700e+001, ...
      -3.9828000e+000, -7.1924300e+001, 6.4194700e+001, -4.7733800e+001,...
      -5.9896000e+000, -2.6282800e+001, -5.9181100e+001, 1.4602800e+001,...
     3 -8.5478000e+001, -5.0490100e+001, 9.2400000e-001, 3.2397800e+001, ... 
     3.0238800e+001, -8.5094900e+001, 6.0119700e+001, -3.6218300e+001,...
     -8.5883000e+000, -5.1971000e+000, 8.1553100e+001 ...
     ];
     for d=1:D
       z(d)=x(d)-offset(d);
    end
    
    fit=-450;
     for d=1:D
       temp=0;
       for j=1:D
        temp=temp+z(j);
       end
       fit=fit+temp*temp;
    end
    
case 53 % Brandeis dice, max entropy
  global s
   s=0;
   h=0;

    for d=1:D 
     s=s+d*x(d); % Weighted probability
    h=h+x(d)*log(x(d)); % -Entropy (to minimise)
    end
 
  	s=abs(s-4.5); % Try to reach  4.5, instead of 3.5 for normal dice
    	p=abs(1-sum(x)); % The six values are probabilities
    	penaltyP=8;
  	penaltyS=1;
  
    fit=h+penaltyP*p +penaltyS*s; 
  %fprintf('\n fit %f :',fit) 
  
  
  case 54 % Graph coloring
  	% fitness = number of invalid pairs of colors
  	fit=0;
    %{
  	for i=1:D-1
  		for j=i+1:D
  			if Gr(i,j)>0 && round(x(i))==round(x(j))
  			fit=fit+1;
  			end
  		end
    end 
    %}
    penalty=D; % Purely empirical
    xInt=uint64(x);
    xInt=compact(xInt);

[valid,nbIncomp]=checkValid(Gr,xInt,1);

if valid
  fit=max(x);
else
    fit=penalty*max(x)+1;
    fit=fit+nbIncomp;
end

   
  case 201901 % CEC 2019 Storn’s Chebyshev Polynomial Fitting Problem
    fit=Chebyshev(x);

    otherwise
        error('\nNo function %i',fun);
end



end

