function [best_x, best_f] =dichoPSO(FuncAddr,fun,quantis,D, LB, UB, FE_max,nRun,...
  dicho, dichDist, dichOption,randOption)
global opt_f
% TEST (see initRand)
reOrder=false;
%{
  Dichotomic PSO  (Maurice Clerc 2023-03)

At each iteration:
1) define two subswarms
2) keep the "best" one
3 replace the other by random particles
There are several options for each step
%}

if dicho
  fprintf('\n dicho dichDist, dichOption, randOption [%i %i %i]',...
    dichDist, dichOption, randOption)
else
  fprintf('\n SPSO 2007')
end
global isOctave % If not Octave, implicitly Matlab
isOctave = exist('OCTAVE_VERSION', 'builtin') ~= 0;

if isOctave
  %pkg load statistics
  %pkg load optim
  more off
  rand ("seed", 123456789) ;
else 
  rng(123456789);
end
%}

global keepAll xAll
xAll=[];
savebf=[];
plotbf=false;
randBest=true; % If true and several equivalent previous best
% select one at random
%{
 if ~exist('dichDist') dichDist=0; end
  if ~exist('dichOption') dichOption=-1; end % See subswarm
  if ~exist('randOption') randOption=1; end
%}

% Setting PSO parameters
w = 1/(2*log(2)); % 0.721
c1 =0.5 + log(2); % 1.193
c2 = c1;

N=ceil(10 + 2 * sqrt(D));	% Swarm size
%N=2*N;
%if mod(N,2)==1 N=N+1; end % Be sure N is even
%if dicho N=ceil(3*N/2); end

% Topology
topo=0; % 0 => random if no improvement
% 1 => Ring
% Suggested K values
switch topo
  case 0
    K=3;
  case 1
    K=1; % For simple ring, K=1
  otherwise
    K=3;
end

%-------------------------------------------------------------------------------
L=topology(N,topo,K);
%bestxNorm=zeros(1,nRun); %just for information
best_f=Inf*ones(1,nRun);

for n=1:nRun
  %fprintf('\n ------- Run %i',n)
  % Initialise position
  FEs=0;
  [x,f,FEs]=initRand(N,LB,UB,opt_f,FuncAddr,fun,quantis);
  %fprintf('\n After init %e',min(f))
  if keepAll
    xAll=[xAll;x];
  end
  
  % Initialize velocities
  for i=1:1:N
    r_p = LB + rand(1,D) .* (UB - LB);
    v(i,:) = 0.5*(r_p - x(i,:));
  end
  
  % Initialize the personal experience
  p_x = x;
  p_f = f;
  
  
  % Index of the global best particle
  [best_f(n), ind] = min(p_f); g=ind(1);
  %fprintf('\n N %i g %i',N,g)
  %fprintf('\n best_f after init %e',best_f(n))
  
  count = 1;
  
  while FEs < FE_max
    for i = 1:N
      % Find the best informant
      MIN = Inf;
      
      for s=1:1:N
        if (L(s,i) == 1)
          if p_f(s) < MIN
            MIN = p_f(s);
            g_best = s;
          end
        end
      end
      
      % Velocity update equation
      v(i,:) = w*v(i,:) + c1*rand(1,D).*(p_x(i,:) - x(i,:))...
        + c2*rand(1,D).*(p_x(g_best,:) - x(i,:));
      
      % Position update equation
      x(i,:) = x(i,:) + v(i,:);
      
      % If discrete, at least on some dimensions
      if  ~isempty(quantis)
        x(i,:)=quantify(x(i,:),quantis);
      end
      
      % Boundary handling
      for j = 1:D
        if x(i,j) > UB(j)
          x(i,j) = UB(j);
          v(i,j) = 0;
          %v(i,j)=-rand()*v(i,j)/2;
        end
        
        if x(i,j) < LB(j)
          x(i,j) = LB(j);
          v(i,j) = 0;
          %v(i,j)=-rand()*v(i,j)/2;
        end
      end %j
      
      if keepAll
        xAll=[xAll;x(i,:)];
      end
      
      % Fitness of particle i
      f(i)=FuncAddr(x(i,:),fun, opt_f); FEs=FEs+1;
      
    end %  for i = 1:N
    %
    %------------------------------------------------ Dichotomy
    if dicho
     % [x,v,f]=dichoSplit(x,v,f,p_x,p_f,g,dichDist,dichOption,randOption,...
     %  LB,UB,quantis,FuncAddr,fun);
     
     % TEST. Worst particle => random
     [~,ind]=max(f);
     wrst=ind(1);
     x(wrst,:)=LB + rand(1,D) .* (UB - LB);
     p_x(wrst,:)=x(wrst,:);
     r_p = LB + rand(1,D) .* (UB - LB);
      %v(wrst,:) =zeros(1,D); 
      v(wrst,:)= 0.5*(r_p - x(i,:));
      f(wrst)=FuncAddr(x(wrst,:),fun, opt_f);  FEs=FEs+1;
      p_f(wrst)=f(wrst);
     %}
    end
    %------------------------------------------------ End of dichotomy
    
     % To avoid stagnation if too many similar f values
    if randBest
      Ind=randperm(N);
      f=f(Ind);
      p_f=p_f(Ind);
      x=x(Ind,:);
      p_x=p_x(Ind,:);
      v=v(Ind,:);
    end
      %}
    % Update personal best
    for i=1:N
      if f(i) <= p_f(i)
        p_x(i,:) = x(i,:);
        p_f(i) = f(i);
      end
    end %  for i = 1:N
    
    % Update global best
    [b_f, ind] = min(p_f); g=ind(1);
    
    if b_f <= best_f(n)
      best_f(n) = b_f;
      count = 0;
    else
      count = count + 1;
    end
    
    if count > 0
      L=topology(N,topo,K);
    end
    if plotbf
      savebf=[savebf best_f(n)];
    end
  end %   while FEs < FE_max
  
  best_x(n,:) = p_x(g,:);
  %fprintf('\n Run %i => %e',n,p_f(g))
  %fprintf('\n  Swarm diameter %e',diameter(x))
  %bestxNorm(n)=norm(best_x(n,:));
  %disp(savebf)
  if plotbf figure ;plot(savebf) ; end
end% for n=1:nRun

if keepAll
  figure
  scatter(xAll(:,1),xAll(:,2),'filled');
end

%fclose('all');
end %PSO

